import 'dart:typed_data';

import 'package:pdfio/pdfio.dart';

void test(String tag,Function tester){
  print(tag);
  tester();
}

void expect(dynamic a,dynamic b){
  if(a==b) {
    print('ok');
  }else{
    print('no');
  }
}

void main() {
  test('PdfDataTypes Bool ', () {
    expect(const PdfBool(true).toString(), 'true');
    expect(const PdfBool(false).toString(), 'false');
  });

  test('PdfDataTypes Num', () {
    expect(const PdfNum(0).toString(), '0');
    expect(const PdfNum(.5).toString(), '0.50000');
    expect(const PdfNum(50).toString(), '50');
    expect(const PdfNum(50.1).toString(), '50.10000');
  });

  test('PdfDataTypes String', () {
    expect(PdfString.fromString('test').toString(), '(test)');
    expect(PdfString.fromString('Zoé').toString(), '(Zoé)');
    expect(PdfString.fromString('\r\n\t\b\f)()(\\').toString(), r'(\r\n\t\b\f\)\(\)\(\\)');
    expect(PdfString.fromString('你好').toList(), <int>[40, 254, 255, 79, 96, 89, 125, 41]);
    expect(PdfString.fromDate(DateTime.fromMillisecondsSinceEpoch(1583606302000)).toString(), '(D:20200307183822Z)');
    expect(PdfString(Uint8List.fromList(const <int>[0, 1, 2, 3, 4, 5, 6]), PdfStringFormat.binary).toString(), '<00010203040506>');
  });

  test('PdfDataTypes Name', () {
    expect(const PdfName('/Hello').toString(), '/Hello');
  });

  test('PdfDataTypes Null', () {
    expect(const PdfNull().toString(), 'null');
  });

  test('PdfDataTypes Indirect', () {
    expect(const PdfIndirect(30, 4).toString(), '30 4 R');
  });

  test('PdfDataTypes Array', () {
    expect(PdfArray().toString(), '[]');
    expect(
      PdfArray(<PdfDataType>[const PdfNum(1), const PdfNum(2)]).toString(),
      '[1 2]',
    );
    expect(
      PdfArray(<PdfDataType>[
        const PdfName('/Name'),
        const PdfName('/Other'),
        const PdfBool(false),
        const PdfNum(2.5),
        const PdfNull(),
        PdfString.fromString('helło'),
        PdfArray(),
        PdfDict(),
      ]).toString(),
      '[/Name/Other false 2.50000 null(þÿ\x00h\x00e\x00l\x01B\x00o)[]<<>>]',
    );
  });

  test('PdfDataTypes Dict', () {
    expect(PdfDict().toString(), '<<>>');

    expect(
      PdfDict(<String, PdfDataType>{
        '/Name': const PdfName('/Value'),
        '/Bool': const PdfBool(true),
        '/Num': const PdfNum(42),
        '/String': PdfString.fromString('hello'),
        '/Null': const PdfNull(),
        '/Indirect': const PdfIndirect(55, 0),
        '/Array': PdfArray(<PdfDataType>[]),
        '/Dict': PdfDict(<String, PdfDataType>{}),
      }).toString(),
      '<</Name/Value/Bool true/Num 42/String(hello)/Null null/Indirect 55 0 R/Array[]/Dict<<>>>>',
    );
  });
}
