import 'package:collection/collection.dart';

///
///  Author: AlmPazel@Gmail.com
///


class Pack {

  static String _str_(Iterable<int> charCodes) => String.fromCharCodes(charCodes);

  List<int> bytes;

  int pos = 0;

  int get byte => bytes[pos];

  bool get isLoop => pos < size;

  int get size => bytes.length;

  String lastIndirect;


  bool get isNumber => '0123456789.'.codeUnits.contains(byte);

  bool get isDelimiter => '<{[(/)]}>'.codeUnits.contains(byte);

  bool get isSpace => 32 == byte; //=' '
  bool get isSlash => 47 == byte; //='/'

  bool get isSpaceAll => [0, 9, 10, 12, 13, 32].contains(byte);

  bool get isSpaceBr => [0, 9, 10, 12, 13].contains(byte);


  bool isContains(String code, int start) {
    var codes = code.codeUnits;
    var end = start + codes.length;
    if (end <= size && ListEquality().equals(codes, bytes.sublist(start, end))) {
      return true;
    }
    return false;
  }

  bool isCmd(String code) {
    if (isContains(code, pos)) {
      pos += code.codeUnits.length;
      return true;
    }
    return false;
  }

  bool isOpen({bool isTrim = true}) {
    if (isTrim) _shift_();
    var _pos = pos;
    if (isCmd('<<') || isCmd('[') || isCmd('(') || isCmd('<')) {
      pos = _pos;
      return true;
    }
    return false;
  }
  var dicts = <dynamic>[];
  Pack(List<int> input) {
    this.bytes = input;
    while (isLoop) {
      var dict = parseNext();
      _shift_();
      if (dict != null) {
        dicts.add(dict);
        continue;
      }
      pos++;
    }
  }

  dynamic parseNext() {
    _shift_();
    var objects = <String, dynamic>{};

    if(_indirect_(()=>isCmd(' obj'))){
      _shift_();
      objects['/Indirect'] = lastIndirect;
      if (isOpen()) {
        var info = parseNext();
        if(info is Map){
          objects.addAll(info);
        }else{
          objects['/Info']=info;
        }
      }
      ///parse stream
      _shift_();
      if (isCmd('stream')) {
        var _pos=pos;
        _while_(() => isCmd('endstream'),shiftTop: true,shiftBot: true);
        objects['/Stream'] = '$_pos,$pos';
      }
      if(isNumber){
        var n=getInteger();
        if(n>=0) objects['/']=n;
      }
      var key = _while_(() => isCmd('endobj'),shiftTop:true);
      if (key.isNotEmpty) objects['/Extra'] = key.length;
      return objects;
    }

    if (isCmd('startxref')) {
      objects['/Type'] = '/StartXRef';
      objects['/'] = getInteger();
      return objects;
    }

    if (isCmd('xref')) {
      objects['/Type'] = '/XRef';
      objects['/'] = parseXrefTable(0, -1);
      return objects;
    }

    if (isCmd('trailer')) {
      objects['/Type'] = '/Trailer';
      objects.addAll(parseNext());
      return objects;
    }

    if (isCmd('%')) {
      objects['/Type'] = '/Comment';
      var key = _while_(() => isSpaceBr);
      objects['/'] = _str_(key);
      return objects;
    }

    if (isCmd('<<')) {
      _shift_();
      while (isLoop) {
        _shift_();
        if (isCmd('>>')) break;
        var key = getString(isKey: true);
        if (isOpen()) {
          objects[key] = parseNext();
        } else {
          objects[key] = getString();
        }
      }
      return objects;
    }

    if (isCmd('[')) {
      _shift_();
      var lt = <dynamic>[];
      while (isLoop) {
        _shift_();
        if (isCmd(']')) break;
        if (isOpen()) {
          lt.add(parseNext());
        } else {
          lt.add(getString());
        }
      }
      return lt;
    }

    if (isCmd('(')) {
      var key = <int>[];
      while (isLoop) {
        if (isOpen(isTrim: false)) {
          key.addAll(parseNext().toString().codeUnits);
        } else {
          if (isCmd(')')) break;
          key.add(byte);
          pos++;
        }
      }
      return '(${_str_(key)})';
    }

    if (isCmd('<')) {
      var key = _while_(() => isCmd('>'));
      return '<${_str_(key)}>';
    }

    return null;
  }

  List<dynamic> parseXrefTable(int idx, int n) {
    var entries = <dynamic>[];
    while (n > 0) {
      var offset = getInteger();
      if (offset == -1) break;
      var gen = getInteger();
      if (gen == -1) break;
      var flag = getString(isKey: true);
      var line = '${idx++},$offset,$gen,$flag';
      entries.add(line);
      n--;
    }
    _shift_();
    if (isNumber) {
      var _pos = pos;
      var idx = getInteger();
      var n = getInteger();
      if (idx >= 0 && n > 0) {
        entries.addAll(parseXrefTable(idx, n));
      } else {
        pos = _pos;
      }
    }
    return entries;
  }

  num getInteger() {
    var _pos=pos;
    var number=_while_(() => !isNumber,shiftTop: true);
    if(number.isEmpty) {
      pos=_pos;
      return -1;
    }
    return num.parse(_str_(number));
  }

  String _getString() {
    _shift_();
    var key = <int>[];
    if (isSlash) {
      key.add(byte);
      pos++;
    }
    key += _while_(() => isSpaceAll || isDelimiter);
    return _str_(key);
  }


  dynamic getString({bool isKey = false}) {
    _shift_();
    if (isKey) return _getString();
    if(_indirect_(()=>isCmd(' R'))){
      return lastIndirect;
    }
    return _getString();
  }

  bool _indirect_(dynamic fun){
    _shift_();
    lastIndirect=null;
    if(isNumber) {
      var _pos=pos;
      var oid=getInteger();
      var gen=getInteger();
      if(oid>=0&&gen>=0&&fun()){
        lastIndirect='$oid $gen R';
        return true;
      }
      pos=_pos;
    }
    return false;
  }

  void _shift_() => _while_(() => !isSpaceAll);

  List<int> _while_(dynamic fun, {bool shiftTop=false,bool shiftBot=false}) {
    if(shiftTop) _shift_();
    var key = <int>[];
    while (isLoop) {
      if (fun()) break;
      key.add(byte);
      pos++;
    }
    if(shiftBot) _shift_();
    return key;
  }
}

