// ignore_for_file: omit_local_variable_types

part of pdfio;

class PdfCatalog extends PdfObject {

  PdfCatalog(
    PdfDocument pdfDocument,
    this.pdfPageList,
    this.names,
  )   : assert(pdfPageList != null),
        assert(names != null),
        super(pdfDocument, '/Catalog');

  final PdfPageList pdfPageList;

  PdfOutline outlines;

  final PdfNames names;

  /// @param os OutputStream to send the object to
  @override
  void _prepare() {
    super._prepare();

    params['/Version'] = PdfName('/${pdfDocument.version}');
    params['/Pages'] = pdfPageList.ref();
    if (outlines != null && outlines.outlines.isNotEmpty) {
      params['/Outlines'] = outlines.ref();
    }
    params['/Names'] = names.ref();

    params['/PageMode'] = PdfName([
      '/UseNone',
      '/UseOutlines',
      '/UseThumbs',
      '/FullScreen'
    ].first);

    final List<PdfAnnotation> widgets = <PdfAnnotation>[];
    for (PdfPage page in pdfDocument.pdfPageList.pages) {
      for (PdfAnnotation annot in page.annotations) {
        if (annot.annot.subtype == '/Widget') {
          widgets.add(annot);
        }
      }
    }

    if (widgets.isNotEmpty) {
      params['/AcroForm'] = PdfDict(<String, PdfDataType>{
        '/Fields': PdfArray.fromObjects(widgets),
      });
    }
  }
}
