part of pdfio;

enum PdfOutlineMode { fitpage, fitrect }

class PdfOutline extends PdfObject {

  PdfOutline(PdfDocument pdfDocument, {this.title, this.dest, this.rect}) : super(pdfDocument, '/Outlines');

  List<PdfOutline> outlines = <PdfOutline>[];

  PdfOutline parent;

  final String title;

  PdfPage dest;

  final PdfRect rect;

  PdfOutlineMode destMode = PdfOutlineMode.fitpage;

  PdfOutline add({String title, PdfPage dest, PdfRect rect}) {
    final PdfOutline outline = PdfOutline(pdfDocument, title: title, dest: dest, rect: rect);
    outline.parent = this;
    return outline;
  }

  @override
  void _prepare() {
    super._prepare();
    if (parent != null) {
      params['/Title'] = PdfString.fromString(title);
      final PdfArray dests = PdfArray();
      dests.add(dest.ref());

      if (destMode == PdfOutlineMode.fitpage) {
        dests.add(const PdfName('/Fit'));
      } else {
        dests.add(const PdfName('/FitR'));
        dests.add(PdfNum(rect.left));
        dests.add(PdfNum(rect.bottom));
        dests.add(PdfNum(rect.right));
        dests.add(PdfNum(rect.top));
      }
      params['/Parent'] = parent.ref();
      params['/Dest'] = dests;

      final int c = descendants();
      if (c > 0) {
        params['/Count'] = PdfNum(-c);
      }

      final int index = parent.getIndex(this);
      if (index > 0) {
        // Now if were not the first, then we have a /Prev node
        params['/Prev'] = parent.getNode(index - 1).ref();
      }

      if (index < parent.getLast()) {
        // We have a /Next node
        params['/Next'] = parent.getNode(index + 1).ref();
      }
    } else {
      // the number of outlines in this document
      // were the top level node, so all are open by default
      params['/Count'] = PdfNum(outlines.length);
    }

    // These only valid if we have children
    if (outlines.isNotEmpty) {
      // the number of the first outline in list
      params['/First'] = outlines[0].ref();

      // the number of the last outline in list
      params['/Last'] = outlines[outlines.length - 1].ref();
    }
  }

  /// This is called by children to find their position in this outlines
  /// tree.
  ///
  /// @param outline [PdfOutline] to search for
  /// @return index within Vector
  int getIndex(PdfOutline outline) => outlines.indexOf(outline);

  /// Returns the last index in this outline
  /// @return last index in outline
  int getLast() => outlines.length - 1;

  /// Returns the outline at a specified position.
  /// @param i index
  /// @return the node at index i
  PdfOutline getNode(int i) => outlines[i];

  /// Returns the total number of descendants below this one.
  /// @return the number of descendants below this one
  int descendants() {
    int c = outlines.length; // initially the number of kids

    // now call each one for their descendants
    for (PdfOutline o in outlines) {
      c += o.descendants();
    }

    return c;
  }
}
